//
//  LoginExample.swift
//  LoginTutorial
//
//  Created by QuentinArguillere on 08/09/2021.
//  Copyright © 2021 BelledonneCommunications. All rights reserved.
//

import linphonesw

class LoginTutorialContext : ObservableObject
{
	var mCore: Core!
	@Published var coreVersion: String = Core.getVersion
	
	/*------------ Login tutorial related variables -------*/
	var mRegistrationDelegate : CoreDelegate!
	@Published var username : String = "1002"
	@Published var passwd : String = "EACE38F5704D4A15BB0B87130AE0E177"
	@Published var domain : String = "s01.startxlabstemxjft.dev.ringplan.com"
	@Published var loggedIn: Bool = false
	@Published var transportType : String = "TCP"
	
	init()
	{
		
		LoggingService.Instance.logLevel = LogLevel.Debug
		
        // Load config
        let configFilename = documentFile("linphonerc222")
        let factoryConfigFilename = bundleFile("linphonerc-factory.pdf")
        
//        let configFilenamePtr: UnsafePointer<Int8> = configFilename.cString(using: String.Encoding.utf8.rawValue)!
//        let factoryConfigFilenamePtr: UnsafePointer<Int8> = factoryConfigFilename.cString(using: String.Encoding.utf8.rawValue)!
        
        try? mCore = Factory.Instance.createCore(configPath: configFilename as String, factoryConfigPath: nil, systemContext: nil)
		
       
            try? mCore.start()
       
       
        
		
		// Create a Core listener to listen for the callback we need
		// In this case, we want to know about the account registration status
		mRegistrationDelegate = CoreDelegateStub(onAccountRegistrationStateChanged: { (core: Core, account: Account, state: RegistrationState, message: String) in
			
			// If account has been configured correctly, we will go through Progress and Ok states
			// Otherwise, we will be Failed.
			NSLog("New registration state is \(state) for user id \( String(describing: account.params?.identityAddress?.asString()))\n")
            print(self.mCore.accountList.count)
            print(core.getCobject)
			if (state == .Ok) {
				self.loggedIn = true
			} else if (state == .Cleared) {
				self.loggedIn = false
			}
		})
		mCore.addDelegate(delegate: mRegistrationDelegate)
	}
	
	func login() {
		
		do {
			// Get the transport protocol to use.
			// TLS is strongly recommended
			// Only use UDP if you don't have the choice
			var transport : TransportType
			if (transportType == "TLS") { transport = TransportType.Tls }
			else if (transportType == "TCP") { transport = TransportType.Tcp }
			else  { transport = TransportType.Udp }
			
			// To configure a SIP account, we need an Account object and an AuthInfo object
			// The first one is how to connect to the proxy server, the second one stores the credentials
			
			// The auth info can be created from the Factory as it's only a data class
			// userID is set to null as it's the same as the username in our case
			// ha1 is set to null as we are using the clear text password. Upon first register, the hash will be computed automatically.
			// The realm will be determined automatically from the first register, as well as the algorithm
			let authInfo = try Factory.Instance.createAuthInfo(username: username, userid: "", passwd: passwd, ha1: "", realm: "", domain: domain)
			
			// Account object replaces deprecated ProxyConfig object
			// Account object is configured through an AccountParams object that we can obtain from the Core
			let accountParams = try mCore.createAccountParams()
			
			// A SIP account is identified by an identity address that we can construct from the username and domain
			let identity = try Factory.Instance.createAddress(addr: String("sip:" + username + "@" + domain))
			try! accountParams.setIdentityaddress(newValue: identity)
			
            let address = try Factory.Instance.createAddress(addr: String("sip:dev.mobile.ringplan.com" ))
            
        
		
			
			// We use the Address object to easily set the transport protocol
			try address.setTransport(newValue: transport)
			try accountParams.setServeraddress(newValue: address)
			// And we ensure the account will start the registration process
			accountParams.registerEnabled = true
			
			// Now that our AccountParams is configured, we can create the Account object
			let account = try mCore.createAccount(params: accountParams)
			
			// Now let's add our objects to the Core
		
			try mCore.addAccount(account: account)
			
			// Also set the newly added account as default
			mCore.defaultAccount = account
            mCore.addAuthInfo(info: authInfo)
            
		} catch { NSLog(error.localizedDescription) }
	}
	
	func unregister()
	{
        
        
        self.mCore.refreshRegisters()

        return
        
		// Here we will disable the registration of our Account
		if let account = mCore.defaultAccount {
			
			let params = account.params
			// Returned params object is const, so to make changes we first need to clone it
			let clonedParams = params?.clone()
			
			// Now let's make our changes
			clonedParams?.registerEnabled = false
			
			// And apply them
			account.params = clonedParams
		}
	}
	func delete() {
        self.mCore.refreshRegisters()

        return
		// To completely remove an Account
		if let account = mCore.defaultAccount {
			mCore.removeAccount(account: account)
			
			// To remove all accounts use
			mCore.clearAccounts()
			
			// Same for auth info
			mCore.clearAllAuthInfo()
		}
	}
    
    fileprivate func documentFile(_ file: NSString) -> NSString {
        let paths = NSSearchPathForDirectoriesInDomains(FileManager.SearchPathDirectory.documentDirectory, FileManager.SearchPathDomainMask.userDomainMask, true)
        
        let documentsPath: NSString = paths[0] as NSString
        return documentsPath.appendingPathComponent(file as String) as NSString
    }
    fileprivate func bundleFile(_ file: NSString) -> NSString{
        return Bundle.main.path(forResource: file.deletingPathExtension, ofType: file.pathExtension)! as NSString
    }
    
    
    
	
}
